from mifiel import Base, Response
import mimetypes
from os.path import basename
import requests
try:
    import simplejson as json
except ImportError:
    import json

class Document(Base):
  def __init__(self, client):
    Base.__init__(self, client, 'documents')

  @staticmethod
  def find(client, doc_id):
    doc = Document(client)
    doc.process_request('get', url=doc.url(doc_id))
    return doc

  @staticmethod
  def all(client):
    base = Document(client)
    response = base.execute_request('get', url=base.url())
    result = []
    for single in response.json():
      obj = Document(client)
      obj.set_data(single)
      result.append(obj)
    return result

  @staticmethod
  def create(client, signatories, **kwargs):
    file = kwargs.get('file')
    dhash = kwargs.get('dhash')
    name = kwargs.get('name')
    if not file and not dhash:
      raise ValueError('Either file or hash must be provided')
    if file and dhash:
      raise ValueError('Only one of file or hash must be provided')
    if dhash and not name:
      raise ValueError('A name is required when using hash')

    data = kwargs.copy()
    for index, item in enumerate(signatories):
      for key, val in item.items():
        data.update(
          {'signatories[' + str(index) + '][' + str(key) + ']': val}
        )

    if 'callback_url' in kwargs: data['callback_url'] = kwargs.get('callback_url')
    if file:
      mimetype = mimetypes.guess_type(file)[0]
      _file = open(file, 'rb')
      file = {'file': (basename(_file.name), _file, mimetype)}
    if dhash:
      data['original_hash'] = data.pop('dhash')

    doc = Document(client)
    doc.process_request('post', data=data, files=file)
    return doc

  def request_signature(self, signer, cc=None):
    path = '{}/{}'.format(self.id, 'request_signature')
    data = {
      'email': signer,
      'cc': cc
    }
    response = self.execute_request('post', url=self.url(path), json=data)
    return response.json()

  @staticmethod
  def delete(client, doc_id):
    base = Document(client)
    response = base.execute_request('delete', url=base.url(doc_id))
    try:
      return response.json()
    except json.JSONDecodeError:
      if response.status_code in [204, 205]:
        # if the response body is empty returns a dictionary with the success response
        return {'status': 'success'}
      else:
        return response.text

  @staticmethod
  def create_from_template(client, args={}):
    call = Base(client, 'templates')
    path = '{}/{}'.format(args['template_id'], 'generate_document')
    response = call.execute_request('post', url=call.url(path), json=args)
    doc = Document(client)
    doc.set_data(response)
    return doc

  @staticmethod
  def create_many_from_template(client, args={}):
    call = Base(client, 'templates')
    url = '{}/{}'.format(args['template_id'], 'generate_documents')
    response = call.execute_request('post', url=call.url(url), json=args)
    return response.json()

  def save_file(self, path):
    url_ = self.url('{}/file').format(self.id)
    response = requests.get(url_, auth=self.client.auth)
    with open(path, 'wb') as file_:
      file_.write(response.content)

  def save_file_signed(self, path):
    url_ = self.url('{}/file_signed').format(self.id)
    response = requests.get(url_, auth=self.client.auth)
    with open(path, 'wb') as file_:
      file_.write(response.content)

  def save_xml(self, path):
    url_ = self.url('{}/xml').format(self.id)
    response = requests.get(url_, auth=self.client.auth)
    with open(path, 'w') as file_:
      file_.write(response.text)



import locale
import threading

from contextlib import contextmanager

LOCALE_LOCK = threading.Lock()


@contextmanager
def setlocale(name):
    with LOCALE_LOCK:
        saved = locale.setlocale(locale.LC_ALL)
        try:
            yield locale.setlocale(locale.LC_ALL, name)
        finally:
            locale.setlocale(locale.LC_ALL, saved)



from .api_auth import RequestsApiAuth

class Client:
  def __init__(self, app_id, secret_key):
    self.sandbox = False
    self.auth = RequestsApiAuth(app_id, secret_key)
    self.base_url = 'https://www.mifiel.com'
    self.timeout = None

  def use_sandbox(self):
    self.sandbox = True
    self.base_url = 'https://sandbox.mifiel.com'

  def set_base_url(self, base_url):
    self.base_url = base_url

  def url(self):
    return self.base_url + '/api/v1/{path}'

  def set_timeout(self, timeout):
    assert type(timeout) == int
    self.timeout = timeout



"""
[ApiAuth](https://github.com/mgomes/api_auth) for python

"""

from .signature import Signature

class ApiAuth():
  def __init__(self, access_id, secret_key):
    self.access_id = access_id
    self.secret_key = secret_key

  def sign(self, request):
    sig = Signature(self.secret_key)
    sig.build(
      method=request.method,
      url=request.url,
      body=request.body,
      content_md5=request.headers.get('content-md5'),
      content_type=request.headers.get('content-type'),
      httpdate=request.headers.get('date')
    )

    request.headers['content-md5'] = sig.content_md5
    request.headers['date'] = sig.httpdate

    auth = 'APIAuth %s:%s' % (self.access_id, sig.signature)
    request.headers['Authorization'] = auth
    return request



from mifiel import Response
import requests

class Base(object):
  def __init__(self, client, path):
    object.__setattr__(self, 'sandbox', False)
    object.__setattr__(self, 'path', path)
    object.__setattr__(self, 'client', client)
    object.__setattr__(self, 'response', Response())
    # initialize id
    self.id = None

  def save(self):
    if self.id is None:
      return False
    self.process_request('put', url=self.url(self.id), data=self.get_data())

  def url(self, path=None):
    p = self.path
    if path:
      p = '{}/{}'.format(p, path)

    return self.client.url().format(path=p)

  def execute_request(self, method, url=None, data=None, json=None, files=None):
    if not url:
      url = self.url()

    kwargs = dict(
      url=url,
      auth=self.client.auth,
      json=json,
    )
    if self.client.timeout:
      kwargs.update({
        'timeout': self.client.timeout
      })

    if method == 'post':
      kwargs.update({
        'data': data,
        'files': files
      })
      response = requests.post(**kwargs)
      if files:
        for file_name in files:
          file = files[file_name]
          if file: file[1].close()
    elif method == 'put':
      response = requests.put(**kwargs)
    elif method == 'get':
      response = requests.get(**kwargs)
    elif method == 'delete':
      response = requests.delete(**kwargs)

    return response

  def process_request(self, method, url=None, data=None, json=None, files=None):
    response = self.execute_request(method, url, data, json, files)
    self.set_data(response)

  def set_data(self, response):
    self.response.set_response(response)

  def get_data(self):
    return self.response.get_response()

  def __setattr__(self, name, value):
    self.response.set(name, value)

  def __getattr__(self, name):
    return self.response.get(name)



"""
[ApiAuth](https://github.com/mgomes/api_auth) for python
Based on https://github.com/pd/httpie-api-auth by Kyle Hargraves
Usage:
import requests
requests.get(url, auth=ApiAuth(app_id, secret_key))

"""
from requests.auth import AuthBase

from .api_auth import ApiAuth

class RequestsApiAuth(AuthBase):
  def __init__(self, access_id, secret_key):
    self.auth = ApiAuth(access_id, secret_key)

  def __call__(self, request):
    return self.auth.sign(request)



import requests

class Response(object):
  def __init__(self):
    object.__setattr__(self, 'datastore', {})

  def set_response(self, response):
    if type(response) is requests.models.Response:
      response.raise_for_status()
      object.__setattr__(self, 'datastore', response.json())
    else:
      object.__setattr__(self, 'datastore', response)

  def get_response(self):
    return self.datastore

  def __setattr__(self, name, value):
    self.set(name, value)

  def __getattr__(self, name):
    return self.get(name)

  def set(self, name, value):
    self.datastore[name] = value

  def get(self, name):
    return self.datastore[name]



import hmac
import base64
import hashlib
import datetime

from mifiel.api_auth.context_manager import setlocale

try:
  from urllib.parse import urlparse
except:
  from urlparse import urlparse

class Signature:
  def __init__(self, secret_key):
    self.secret_key = secret_key.encode('ascii')

  def build(self, method, url, body, content_md5=None, content_type=None, httpdate=None):
    method = method.upper()

    if not content_type:
      content_type = ''

    self.content_md5 = content_md5
    if not self.content_md5:
      m = hashlib.md5()
      if not body:
        body = ''
      if isinstance(body, str):
        try:
          m.update(body.encode('ascii'))
        # For Python 2.7
        except(UnicodeDecodeError):
          m.update(body)
      else:
        m.update(body)

      self.content_md5 = base64.b64encode(m.digest()).decode()

    self.httpdate = httpdate
    if not self.httpdate:
      now = datetime.datetime.utcnow()
      with setlocale('en_US.UTF-8'):
        self.httpdate = now.strftime('%a, %d %b %Y %H:%M:%S GMT')

    url = urlparse(url)
    path = url.path
    if url.query:
      path = path + '?' + url.query

    rep = (method, content_type, self.content_md5, path, self.httpdate)
    self.canonical_string = '%s,%s,%s,%s,%s' % rep

    digest = hmac.new(
      self.secret_key,
      self.canonical_string.encode('ascii'),
      hashlib.sha1
    ).digest()
    self.signature = base64.b64encode(digest).rstrip().decode()



from setuptools import setup

try:
  long_description = open('README.txt').read()
except(IOError, ImportError):
  long_description = open('README.md').read()

setup(
  name='mifiel',
  version='0.0.10',
  description='Python API Client library for Mifiel.com',
  long_description=long_description,
  url='http://github.com/mifiel/python-api-client',
  download_url='https://github.com/Mifiel/python-api-client/tarball/v0.0.10',
  author='Genaro Madrid',
  author_email='genmadrid@gmail.com',
  license='MIT',
  test_suite='nose2.collector.collector',
  packages=['mifiel', 'mifiel.api_auth'],
  install_requires=[
    'requests'
  ],
  include_package_data=True,
  zip_safe=False
)



from mifiel import Base, Document
import mimetypes
from os.path import basename
import requests

class Template(Base):
  def __init__(self, client):
    Base.__init__(self, client, 'templates')

  @staticmethod
  def find(client, template_id):
    template = Template(client)
    template.process_request('get', url=template.url(template_id))
    return template

  @staticmethod
  def all(client):
    base = Template(client)
    response = base.execute_request('get', url=base.url())
    result = []
    for single in response.json():
      obj = Template(client)
      obj.set_data(single)
      result.append(obj)
    return result

  @staticmethod
  def create(client, name, content, description=None, header=None, footer=None):
    template = Template(client)
    data = {
      'name': name,
      'description': description,
      'header': header,
      'content': content,
      'footer': footer
    }
    template.process_request('post', data=data)
    return template

  @staticmethod
  def delete(client, template_id):
    base = Template(client)
    response = base.execute_request('delete', url=base.url(template_id))
    return response.json()
